@@ -366,6 +366,22 @@ impl InsertableJob for CleanupFinishedCompatSessionsJob {
     const QUEUE_NAME: &'static str = "cleanup-finished-compat-sessions";
 }
 
+/// Cleanup finished OAuth 2.0 sessions
+#[derive(Serialize, Deserialize, Debug, Clone, Default)]
+pub struct CleanupFinishedOAuth2SessionsJob;
+
+impl InsertableJob for CleanupFinishedOAuth2SessionsJob {
+    const QUEUE_NAME: &'static str = "cleanup-finished-oauth2-sessions";
+}
+
+/// Cleanup finished user/browser sessions
+#[derive(Serialize, Deserialize, Debug, Clone, Default)]
+pub struct CleanupFinishedUserSessionsJob;
+
+impl InsertableJob for CleanupFinishedUserSessionsJob {
+    const QUEUE_NAME: &'static str = "cleanup-finished-user-sessions";
+}
+
 /// Cleanup old OAuth 2.0 authorization grants
 #[derive(Serialize, Deserialize, Debug, Clone, Default)]
 pub struct CleanupOAuthAuthorizationGrantsJob;
@@ -611,3 +627,27 @@ pub struct PruneStalePolicyDataJob;
 impl InsertableJob for PruneStalePolicyDataJob {
     const QUEUE_NAME: &'static str = "prune-stale-policy-data";
 }
+
+/// Cleanup IP addresses from inactive OAuth 2.0 sessions
+#[derive(Serialize, Deserialize, Debug, Clone, Default)]
+pub struct CleanupInactiveOAuth2SessionIpsJob;
+
+impl InsertableJob for CleanupInactiveOAuth2SessionIpsJob {
+    const QUEUE_NAME: &'static str = "cleanup-inactive-oauth2-session-ips";
+}
+
+/// Cleanup IP addresses from inactive compat sessions
+#[derive(Serialize, Deserialize, Debug, Clone, Default)]
+pub struct CleanupInactiveCompatSessionIpsJob;
+
+impl InsertableJob for CleanupInactiveCompatSessionIpsJob {
+    const QUEUE_NAME: &'static str = "cleanup-inactive-compat-session-ips";
+}
+
+/// Cleanup IP addresses from inactive user/browser sessions
+#[derive(Serialize, Deserialize, Debug, Clone, Default)]
+pub struct CleanupInactiveUserSessionIpsJob;
+
+impl InsertableJob for CleanupInactiveUserSessionIpsJob {
+    const QUEUE_NAME: &'static str = "cleanup-inactive-user-session-ips";
+}
