@@ -135,6 +135,8 @@ pub async fn init(
         .register_handler::<mas_storage::queue::CleanupConsumedOAuthRefreshTokensJob>()
         .register_handler::<mas_storage::queue::CleanupUserRegistrationsJob>()
         .register_handler::<mas_storage::queue::CleanupFinishedCompatSessionsJob>()
+        .register_handler::<mas_storage::queue::CleanupFinishedOAuth2SessionsJob>()
+        .register_handler::<mas_storage::queue::CleanupFinishedUserSessionsJob>()
         .register_handler::<mas_storage::queue::CleanupOAuthAuthorizationGrantsJob>()
         .register_handler::<mas_storage::queue::CleanupOAuthDeviceCodeGrantsJob>()
         .register_handler::<mas_storage::queue::CleanupUserRecoverySessionsJob>()
@@ -156,6 +158,9 @@ pub async fn init(
         .register_handler::<mas_storage::queue::ExpireInactiveOAuthSessionsJob>()
         .register_handler::<mas_storage::queue::ExpireInactiveUserSessionsJob>()
         .register_handler::<mas_storage::queue::PruneStalePolicyDataJob>()
+        .register_handler::<mas_storage::queue::CleanupInactiveOAuth2SessionIpsJob>()
+        .register_handler::<mas_storage::queue::CleanupInactiveCompatSessionIpsJob>()
+        .register_handler::<mas_storage::queue::CleanupInactiveUserSessionIpsJob>()
         .register_deprecated_queue("cleanup-expired-tokens")
         .add_schedule(
             "cleanup-revoked-oauth-access-tokens",
@@ -187,6 +192,18 @@ pub async fn init(
             "0 40 * * * *".parse()?,
             mas_storage::queue::CleanupFinishedCompatSessionsJob,
         )
+        .add_schedule(
+            "cleanup-finished-oauth2-sessions",
+            // Run this job every hour
+            "0 42 * * * *".parse()?,
+            mas_storage::queue::CleanupFinishedOAuth2SessionsJob,
+        )
+        .add_schedule(
+            "cleanup-finished-user-sessions",
+            // Run this job every hour
+            "0 44 * * * *".parse()?,
+            mas_storage::queue::CleanupFinishedUserSessionsJob,
+        )
         .add_schedule(
             "cleanup-oauth-authorization-grants",
             // Run this job every hour
@@ -211,16 +228,12 @@ pub async fn init(
             "0 57 * * * *".parse()?,
             mas_storage::queue::CleanupUserEmailAuthenticationsJob,
         )
-        // This job is currently disabled, as it needs a database backfill to
-        // happen, which will happen in the next release. Some context in
-        // https://github.com/element-hq/matrix-authentication-service/issues/5435
-        //
-        //.add_schedule(
-        //    "cleanup-upstream-oauth-sessions",
-        //    // Run this job every hour
-        //    "0 58 * * * *".parse()?,
-        //    mas_storage::queue::CleanupUpstreamOAuthSessionsJob,
-        //)
+        .add_schedule(
+            "cleanup-upstream-oauth-sessions",
+            // Run this job every hour
+            "0 58 * * * *".parse()?,
+            mas_storage::queue::CleanupUpstreamOAuthSessionsJob,
+        )
         .add_schedule(
             "cleanup-upstream-oauth-links",
             // Run this job every hour
@@ -250,6 +263,24 @@ pub async fn init(
             // Run once a day
             "0 0 2 * * *".parse()?,
             mas_storage::queue::PruneStalePolicyDataJob,
+        )
+        .add_schedule(
+            "cleanup-inactive-oauth2-session-ips",
+            // Run this job every hour
+            "0 46 * * * *".parse()?,
+            mas_storage::queue::CleanupInactiveOAuth2SessionIpsJob,
+        )
+        .add_schedule(
+            "cleanup-inactive-compat-session-ips",
+            // Run this job every hour
+            "0 47 * * * *".parse()?,
+            mas_storage::queue::CleanupInactiveCompatSessionIpsJob,
+        )
+        .add_schedule(
+            "cleanup-inactive-user-session-ips",
+            // Run this job every hour
+            "0 48 * * * *".parse()?,
+            mas_storage::queue::CleanupInactiveUserSessionIpsJob,
         );
 
     Ok(worker)
